@@ -27,6 +27,8 @@
 #define GNOME_DESKTOP_USE_UNSTABLE_API 1
 #include <libgnome-desktop/gnome-wall-clock.h>
 
+static const char *output_dir = NULL;
+
 typedef enum {
   SNAPSHOT_WINDOW,
   SNAPSHOT_DRAW
@@ -39,14 +41,16 @@ static char *
 get_output_file (const char *test_file,
                  const char *extension)
 {
-  const char *output_dir = g_get_tmp_dir ();
   char *result, *base;
 
   base = g_path_get_basename (test_file);
   if (g_str_has_suffix (base, ".ui"))
     base[strlen (base) - strlen (".ui")] = '\0';
 
-  result = g_strconcat (output_dir, G_DIR_SEPARATOR_S, base, extension, NULL);
+  if (output_dir)
+    result = g_strconcat (output_dir, G_DIR_SEPARATOR_S, "..", G_DIR_SEPARATOR_S, "meson-logs", G_DIR_SEPARATOR_S, base, extension, NULL);
+  else
+    result = g_strconcat (g_get_tmp_dir (), G_DIR_SEPARATOR_S, base, extension, NULL);
   g_free (base);
 
   return result;
@@ -435,14 +439,17 @@ test_ui_file (GFile         *file,
   GnomeWallClock *clock;
   GDateTime *datetime;
   char *str;
+  locale_t loc, previous_locale;
 
   ui_file = g_file_get_path (file);
 
   locale = get_locale_for_file (ui_file);
   g_assert (locale);
-  setlocale (LC_ALL, locale);
+  loc = newlocale (LC_ALL_MASK, locale, (locale_t) 0);
+  previous_locale = uselocale (loc);
+  g_assert_true (previous_locale != (locale_t) 0);
 
-  clock = gnome_wall_clock_new();
+  clock = gnome_wall_clock_new ();
   datetime = g_date_time_new_local (2014, 5, 28, 23, 59, 59);
   str = gnome_wall_clock_string_for_datetime (clock,
 					      datetime,
@@ -452,6 +459,9 @@ test_ui_file (GFile         *file,
   g_date_time_unref (datetime);
   g_object_unref (clock);
 
+  uselocale (previous_locale);
+  freelocale (loc);
+
   provider = add_extra_css (ui_file, ".css");
 
   ui_image = snapshot_ui_file (ui_file, str);
@@ -565,17 +575,26 @@ main (int argc, char **argv)
   const char *basedir;
   GFile *file;
 
+  /* unset environment variables that can interfere with our tests */
+  unsetenv("LANG");
+  unsetenv("LANGUAGE");
+  unsetenv("LC_ALL");
+  unsetenv("LC_TIME");
+
   /* I don't want to fight fuzzy scaling algorithms in GPUs,
    * so unless you explicitly set it to something else, we
    * will use Cairo's image surface.
    */
   g_setenv ("GDK_RENDERING", "image", FALSE);
 
-  setlocale (LC_ALL, "");
   g_test_init (&argc, &argv, NULL);
   gtk_init (&argc, &argv);
 
-  basedir = INSTALLED_TEST_DIR;
+  basedir = g_getenv ("G_TEST_SRCDIR");
+  if (basedir == NULL)
+    basedir = INSTALLED_TEST_DIR;
+  output_dir = g_getenv ("G_TEST_BUILDDIR");
+
   file = g_file_new_for_commandline_arg (basedir);
   add_test_for_file (file, NULL);
   g_object_unref (file);
